<?php

namespace JugoPlugin\Classes;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Api {
	protected $namespace;
	protected $rest_base;

	public function __construct() {
		$this->namespace = 'jugo/v1';
		$this->rest_base = '/settings';
	}

	public function init() {
		$this->register_routes();
	}

	public function register_routes() {
		add_action( 'rest_api_init', function () {
			register_rest_route( $this->namespace, "$this->rest_base/ping", [
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'ping' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
			] );
			register_rest_route( $this->namespace, "$this->rest_base/auth", [
				[
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => [ $this, 'create_api_key' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'get_api_key' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				]
			] );
			register_rest_route( $this->namespace, "$this->rest_base/boxes", [
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'get_boxes' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
				[
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => [ $this, 'create_boxes' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
			] );
			register_rest_route( $this->namespace, "$this->rest_base/boxes/(?P<box_id>\d+)", [
				[
					'methods'             => \WP_REST_Server::DELETABLE,
					'callback'            => [ $this, 'delete_boxes' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				]
			] );
			register_rest_route( $this->namespace, "$this->rest_base/pickup_addresses", [
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'get_pickup_addresses' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
				[
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => [ $this, 'create_pickup_addresses' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
			] );
			register_rest_route( $this->namespace, "$this->rest_base/pickup_addresses/(?P<address_id>\d+)", [
				[
					'methods'             => \WP_REST_Server::DELETABLE,
					'callback'            => [ $this, 'delete_pickup_addresses' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				]
			] );
			register_rest_route( $this->namespace, "$this->rest_base/optional_services", [
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'get_optional_services' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
				[
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => [ $this, 'save_optional_services' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
			] );
			register_rest_route( $this->namespace, "$this->rest_base/rates_view", [
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'get_rates_view' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
				[
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => [ $this, 'save_rates_view' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
			] );
			register_rest_route( $this->namespace, "$this->rest_base/general", [
				[
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => [ $this, 'get_general' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
				[
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => [ $this, 'save_general' ],
					'permission_callback' => [ $this, 'get_route_access' ]
				],
			] );
		} );
	}

	public function get_route_access( $request ) {
		return true;
	}

	/**
	 * Test Function
	 */
	public function ping( $request ) {
		$response = [
			'name' => 'pong'
		];

		return rest_ensure_response( $response );
	}

	/**
	 * Save API Key
	 */
	public function create_api_key( $request ) {
		$api_key = isset( $request['api_key'] ) ? sanitize_text_field( $request['api_key'] ) : '';
		// Save API Key into DB
		update_option( 'jugo_settings_api_key', $api_key );

		$response = [
			'api_key' => get_option( 'jugo_settings_api_key' )
		];

		return rest_ensure_response( $response );
	}

	/**
	 * Get API Key
	 */
	public function get_api_key( $request ) {
		$response = [
			'api_key' => get_option( 'jugo_settings_api_key' )
		];

		return rest_ensure_response( $response );
	}

	/**
	 * Save/Update Boxes
	 */
	function create_boxes( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_boxes';

		$boxes_data = $request->get_json_params();


		foreach ( $boxes_data as $box_data ) {
			$box_id             = isset( $box_data['id'] ) ? intval( $box_data['id'] ) : 0;
			$box_data['lato_1'] = $box_data['lato_1'] / 100;
			$box_data['lato_2'] = $box_data['lato_2'] / 100;
			$box_data['lato_3'] = $box_data['lato_3'] / 100;

			if ( $box_id > 0 ) {
				if ( $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $table_name WHERE id = %d", $box_id ) ) > 0 ) {
					$wpdb->update( $table_name, $box_data, [ 'id' => $box_id ] );
				}
			} else {
				$wpdb->insert( $table_name, $box_data );
			}
		}

		return rest_ensure_response( [ 'message' => 'All boxes saved/updated successfully.' ] );
	}

	/**
	 * Delete Boxes
	 */
	function delete_boxes( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_boxes';

		$box_id = $request['box_id'];

		if ( $box_id ) {
			$wpdb->delete( $table_name, [ 'id' => $box_id ] );

			return rest_ensure_response( [ 'message' => 'Box deleted successfully.' ] );
		} else {
			return rest_ensure_response( [ 'error' => 'Box ID is required.' ] );
		}
	}

	/**
	 * Get Boxes
	 */
	function get_boxes() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_boxes';
		$results    = $wpdb->get_results( "SELECT * FROM $table_name" );

		return rest_ensure_response( $results );
	}

	/**
	 * Save/Update Pickup Addresses
	 */
	function create_pickup_addresses( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_pickup_addresses';

		$addresses_data = $request->get_json_params();

		foreach ( $addresses_data as $address_data ) {
			$address_id = isset( $address_data['id'] ) ? intval( $address_data['id'] ) : 0;

			if ( $address_id > 0 ) {
				if ( $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $table_name WHERE id = %d", $address_id ) ) > 0 ) {
					// Se esiste, aggiorno l'indirizzo anche in JUGO
					$request = wp_remote_request( JUGO_BASE_URL . '/api/v2/location/shipping/' . $address_data['jugo_id'], [
						'method'  => 'PUT',
						'body'    => [
							"save"             => true,
							"full_name"        => $address_data['name'],
							"contact"          => null,
							"address"          => $address_data['address'],
							"zip"              => null,
							"city"             => null,
							"province"         => null,
							"zip_id"           => $address_data['zip_code_id'],
							"city_id"          => $address_data['city_id'],
							"province_id"      => $address_data['province_id'],
							"region_id"        => $address_data['region_id'],
							"country_id"       => $address_data['country_id'],
							"directional_info" => null,
							"email"            => $address_data['email'],
							"phone"            => $address_data['phone']
						],
						'headers' => [
							'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' )
						]
					] );

					// Aggiorno indirizzo in WP
					$wpdb->update( $table_name, $address_data, [
						'id'      => $address_id,
						'jugo_id' => $address_data['jugo_id']
					] );
				}
			} else {
				// Se non esiste, salvo indirizzo in JUGO
				$request = wp_remote_post( JUGO_BASE_URL . '/api/v2/location/shipping', [
					'body'    => [
						"save"             => true,
						"full_name"        => $address_data['name'],
						"contact"          => null,
						"address"          => $address_data['address'],
						"zip"              => null,
						"city"             => null,
						"province"         => null,
						"zip_id"           => $address_data['zip_code_id'],
						"city_id"          => $address_data['city_id'],
						"province_id"      => $address_data['province_id'],
						"region_id"        => $address_data['region_id'],
						"country_id"       => $address_data['country_id'],
						"directional_info" => null,
						"email"            => $address_data['email'],
						"phone"            => $address_data['phone']
					],
					'headers' => [
						'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' )
					]
				] );

				$response = json_decode( wp_remote_retrieve_body( $request ) );
				// Salvo i dati in WP aggiungendo l'ID che viene restituito dopo la creazione in JUGO
				$address_data['jugo_id'] = $response->id;
				$wpdb->insert( $table_name, $address_data );
			}
		}

		return rest_ensure_response( [ 'message' => 'All pickup addresses saved/updated successfully.' ] );
	}

	/**
	 * Delete Pickup Addresses
	 */
	function delete_pickup_addresses( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_pickup_addresses';

		$address_data = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE id = %d", $request['address_id'] ) );

		if ( $request['address_id'] ) {
			wp_remote_request( JUGO_BASE_URL . '/api/v2/location/shipping/' . $address_data->jugo_id, [
				'method'  => 'DELETE',
				'headers' => [
					'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' )
				]
			] );

			$wpdb->delete( $table_name, [ 'id' => $request['address_id'] ] );

			return rest_ensure_response( [ 'message' => 'Address deleted successfully.' ] );
		} else {
			return rest_ensure_response( [ 'error' => 'Address ID is required.' ] );
		}
	}

	/**
	 * Get Pickup Addresses
	 */
	function get_pickup_addresses() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_pickup_addresses';
		$results    = $wpdb->get_results( "SELECT * FROM $table_name" );

		return rest_ensure_response( $results );
	}

	/**
	 * Get Optional Services
	 */
	function get_optional_services() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_optional_services';

		$services_data = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

		if ( $services_data ) {
			$services_data->mark      = ! ! $services_data->mark;
			$services_data->ensurance = ! ! $services_data->ensurance;
			$services_data->priority  = ! ! $services_data->priority;


			return rest_ensure_response( $services_data );
		}

		return rest_ensure_response( [] );

	}

	/**
	 * Save/Update Optional Services
	 */
	function save_optional_services( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_optional_services';

		$services_data = $request->get_json_params();

		// Default values
		$default_data = [
			'mark'       => false,
			'mark_value' => 0,
			'ensurance'  => false,
			'priority'   => false
		];

		$services_data = wp_parse_args( $services_data, $default_data );

		$existing_row = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

		if ( $existing_row ) {
			$wpdb->update( $table_name, $services_data, [ 'id' => 1 ] );
		} else {
			$wpdb->insert( $table_name, array_merge( [ 'id' => 1 ], $services_data ) );
		}

		return rest_ensure_response( [ 'message' => 'Optional services updated/created successfully.' ] );
	}

	/**
	 * Get Rates View
	 */
	function get_rates_view() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_rates_view';

		$services_data = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

		if ( $services_data ) {
			return rest_ensure_response( $services_data );
		}

		return rest_ensure_response( [] );
	}

	/**
	 * Save/Update Rates View
	 */
	function save_rates_view( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_rates_view';

		$services_data = $request->get_json_params();

		// Default values
		$default_data = [
			'rate_type'   => null,
			'unique_type' => null,
		];

		$services_data = wp_parse_args( $services_data, $default_data );

		$existing_row = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

		if ( $existing_row ) {
			$wpdb->update( $table_name, $services_data, [ 'id' => 1 ] );
		} else {
			$wpdb->insert( $table_name, array_merge( [ 'id' => 1 ], $services_data ) );
		}

		return rest_ensure_response( [ 'message' => 'Rates View updated/created successfully.' ] );
	}

	/**
	 * Get General
	 */
	function get_general() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_general';

		$services_data = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

		if($services_data) {
			return rest_ensure_response( $services_data );
		}

		return rest_ensure_response( [] );
	}

	/**
	 * Save/Update Rates View
	 */
	function save_general( $request ) {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_general';

		$services_data = $request->get_json_params();

		// Default values
//		$default_data = [
//			'shipping_type'        => 'free',
//			'free_min_value'       => null,
//			'products_box_id'      => null,
//			'products_pickup_id'   => null,
//			'categories_box_id'    => null,
//			'categories_pickup_id' => null,
//			'recharge_percentage' => 0
//		];

		$services_data = wp_parse_args( $services_data );

		$existing_row = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

		if ( $existing_row ) {
			$wpdb->update( $table_name, $services_data, [ 'id' => 1 ] );
		} else {
			$wpdb->insert( $table_name, array_merge( [ 'id' => 1 ], $services_data ) );
		}

		return rest_ensure_response( [ 'message' => 'General updated/created successfully.' ] );
	}
}